import os

import setuptools
from setuptools import find_packages
from setuptools.command.build_py import build_py as build_py_orig


def generate_man1_pages(share_path, docs):
    data_files = []

    for path, _, files in os.walk(docs):
        list_entry = (share_path, [os.path.join(path, f) for f in files if f.endswith(".1")])
        data_files.append(list_entry)

    return data_files


def generate_man5_pages(share_path, docs):
    data_files = []

    for path, _, files in os.walk(docs):
        list_entry = (share_path, [os.path.join(path, f) for f in files if f.endswith(".5")])
        data_files.append(list_entry)

    return data_files


def generate_ramalama_conf(share_path, docs):
    data_files = []

    for path, _, files in os.walk(docs):
        list_entry = (share_path, [os.path.join(path, f) for f in files if f.endswith(".conf")])
        data_files.append(list_entry)

    return data_files


def generate_completions(share_path, completions):
    data_files = []

    def remove_prefix(s, prefix):
        if s.startswith(prefix):
            length = len(prefix) + 1
            return s[length:]
        else:
            return s

    for path, _, files in os.walk(completions):
        if len(files) == 0:
            continue
        list_entry = (
            os.path.join(share_path, remove_prefix(path, completions)),
            [os.path.join(path, f) for f in files],
        )
        data_files.append(list_entry)
    return data_files


class build_py(build_py_orig):
    def find_package_modules(self, package, package_dir):
        modules = super().find_package_modules(package, package_dir)
        return [(pkg, mod, file) for (pkg, mod, file) in modules]


setuptools.setup(
    name="ramalama",
    version="0.7.3",
    packages=find_packages(),
    cmdclass={"build_py": build_py},
    scripts=["bin/ramalama"],
    data_files=[("share/ramalama", ["shortnames/shortnames.conf"])]
    + generate_completions("share", "completions")
    + generate_ramalama_conf("share/ramalama", "docs")
    + generate_man1_pages("share/man/man1", "docs")
    + generate_man5_pages("share/man/man5", "docs")
    + [
        (
            "libexec/ramalama",
            [
                "libexec/ramalama/ramalama-client-core",
                "libexec/ramalama/ramalama-run-core",
                "libexec/ramalama/ramalama-serve-core",
            ],
        )
    ],
)
